###########
#
# Rdaemon start script
#
#   Hans-Jörg Bibiko - bibiko@eva.mpg.de
#
###########

######### global variables #########

if [ "$TM_RdaemonRAMDRIVE" == "1" ]; then
	RDRAMDISK="/tmp/TMRramdisk1"
else
	RDRAMDISK="$HOME"/Rdaemon
fi

RDHOME="$HOME/Rdaemon"
RAMSIZE=${TM_RdaemonRAMSIZE:-50}


######### functions #########

runs() {
	RUBYRUN=$(ps xw | grep '[0-9] ruby.*Rdaemon')
	RRUN=$(ps aw | grep '[0-9] /Lib.*TMRdaemon')
	if [ ${#RRUN} == 0 -a ${#RUBYRUN} == 0 ]; then
		return 0
	else
		return 1
	fi
}

######### script begin #########

#check whether R is already runing
runs
if [ $? == 1 ]; then
	echo -en "Rdaemon is already running!"
	exit 206
fi

#start X11
export DISPLAY=:0.0
#open -a X11
#osascript <<-AS &>/dev/null
# 	tell application "TextMate"
# 		activate
# 	end tell
#AS

#test for ram drive
if [ "$TM_RdaemonRAMDRIVE" == "1" ]; then
	RES=""
	if [ -d "$RDRAMDISK" ]; then
		echo -n "a" > "$RDRAMDISK"/test.txt
		RES=$(cat "$RDRAMDISK"/test.txt)
	fi
	if [ "$RES" != "a" ]; then #create ram drive
		ramDiskSize=`echo "${RAMSIZE}*2048" | bc`
		ramDiskPath=`hdid -nomount ram://${ramDiskSize}`
		#save /dev/disk? for detaching
		echo -n "$ramDiskPath" > ~/Rdaemon/daemon/ramDiskPath
		newfs_hfs -v 'TMRdaemon' $ramDiskPath
#		if [ -d "$RDRAMDISK" ]; then
#			RES=$(rmdir "$RDRAMDISK")
#			[[ ! -z "$RES" ]] && echo "Please delete $RDRAMDISK manually!" && exit 206
#		fi
		[[ ! -d "$RDRAMDISK" ]] && mkdir "$RDRAMDISK"
		mount -t hfs $ramDiskPath "$RDRAMDISK"
	fi	
	echo -n "a" > "$RDRAMDISK"/test.txt 2> /dev/null
	RES=$(cat "$RDRAMDISK"/test.txt 2> /dev/null) 2> /dev/null
	[[ "$RES" != "a" ]] && echo "Please check RAM drive manually!" && exit 206
	rm "$RDRAMDISK"/test.txt
fi

#create plot directory
[[ ! -d "$RDHOME/plots" ]] && mkdir "$RDHOME/plots"

#create history directory
[[ ! -d "$RDHOME/history" ]] && mkdir "$RDHOME/history"

#create the named pipe /tmp/r_in
if [ ! -e "$RDHOME"/r_in ]; then
	mkfifo "$RDHOME"/r_in
else
	if [ ! -p "$RDHOME"/r_in ]; then
		rm "$RDHOME"/r_in
		mkfifo "$RDHOME"/r_in
	fi
fi
sleep 0.5

#delete r_out and r_tmp
[[ -f "$RDRAMDISK"/r_out ]] && rm "$RDRAMDISK"/r_out
[[ -f "$RDRAMDISK"/r_tmp ]] && rm "$RDRAMDISK"/r_tmp

#copy console.r if not exists 
if [ ! -f "$RDHOME"/console.r ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/console.r "$RDHOME"/console.r
fi
#copy Rsession TMProject if not exists 
if [ ! -f "$RDHOME"/Rsession.tmproj ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/Rsession.tmproj "$RDHOME"/Rsession.tmproj
fi
#copy startOptions.R if not exists 
if [ ! -f "$RDHOME"/startOptions.R ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/startOptions.R "$RDHOME"/startOptions.R
fi
#copy Rhistory.txt if not exists 
if [ ! -f "$RDHOME"/history/Rhistory.txt ]; then
	cp "$TM_BUNDLE_SUPPORT"/bin/Rhistory.txt "$RDHOME"/history/Rhistory.txt
fi

#remove duplicated lines of /Rhistory.txt
cat "$RDHOME"/history/Rhistory.txt | uniq > "$RDHOME"/history/Rhistory.temp
rm "$RDHOME"/history/Rhistory.txt
cat "$RDHOME"/history/Rhistory.temp > "$RDHOME"/history/Rhistory.txt
rm "$RDHOME"/history/Rhistory.temp

#start Rdaemon
"$RDHOME"/daemon/startR.sh &> /dev/null &

#reset history counter
echo -n 0 > "$HOME"/Rdaemon/history/Rhistcounter.txt

#safety counter
SAFECNT=0
#wait until r_out is created (or not) by the Rdaemon
while [ ! -f "$RDRAMDISK"/r_out ]
do
	SAFECNT=$(($SAFECNT+1))
	if [ $SAFECNT -gt 50000 ]; then
		echo -en "Start failed! No response from Rdaemon!"
		exit 206
	fi
done

#wait for Rdaemon's output is ready
S=0
CNT=1
while [ "$S" == 0 ]
do
	ST=$(tail -n 1 "$RDRAMDISK"/r_out )
	if [ "$ST" == "> " ]; then
		S=1
	fi
	CNT=$(($CNT+15))
	echo "$CNT Rdaemon is starting ..."; sleep .5
done|CocoaDialog progressbar --title "Rdaemon"

runs
if [ $? == 1 ]; then
	echo -en "> "
else
	echo "While starting Rdaemon was shutdown unexpectedly!"
	exit 206
fi
